// C Source File
// Created 11.02.2006; 16:30:33

/***********************************************************************
*
*  fmanage
* =========
*
* Manage a binary-file
*
* Manage a file with structures.
* Add, remove and edit such a structure
* Should work with AMS 1.00
*
*
*  example:
* ==========
*
* header.h:
* ---------
* typedef struct
*		{
*			int dummy;
*
*		} YOURSTRUCT;
*
*
* main.c:
* -------
* 1. FILE *f = NULL;
*	2. YOURSTRUCT *ys = NULL;
* 3. fmanage(&f, "yourfile", 1, "file", NULL, sizeof(YOURSTRUCT), 0, FMF_NEW_FILE, FMF_NEW_NO_OVERWR));
* 4. fmanage(&f, "yourfile", 1, "file", ys, sizeof(YOURSTRUCT), 0, FMF_ADD, 0));
* 5. fclose(f);
*
* description:
* ------------
* 1. Define the structure (for example in your header file)
* 2. Make a pointer to your structure (for exapmle *ys) (don't forget to allocate memory)
* 3. Open a new file and add the used file structure
* 4. Open a file in binary-mode and add your structure to it.
* 5. Close your file (if not already closed with fmanage, see options for more information)
*
*
*
*  Return:
* =========
*  NumberOfBlocks: Success (when action == FMF_SEARCH, 0 means the data has not been found or when action == FMF_NEW_FILE you choosed not to overwrite the old file)
* -1: Error while operatin with the file (fopen/fclose/fseek/archive/...)
* -2: Error while reading
* -3: Error while writing
* -4: Wrong file format (the size of the stored structure is not the same like the size of the data structure or the version number of the file is not correct)
* -5: Wrong range of a variable
* -6: A part of the function has been disabled but still used (check up fmanage_user.h for more information)
* -7: Memory error
*
* -10: File is locked
*
*
*
*
*
*  Example of a bit-test
* =======================
* if (action & FMF_LOAD == FMF_LOAD)
*
*   1111 1111 -> example for "action"
* & 0000 1000 -> FMF_LOAD
* = 0000 1000 => FMF_LOAD
*
***********************************************************************/

#include <tigcclib.h>
#include "fmanage.h"

// Some prototypes of seperated functions.
int check_excluded_functions(unsigned short action, unsigned short option);
#ifndef	FMF_EXCLUDE_RETURN_FINFO
long int return_fileinfo(FMFILEINFO *finfo, unsigned short action, unsigned short option);
#endif
#ifndef FMF_EXCLUDE_CHECK_FILEINFO
int check_fileinfo(FMFILEINFO *finfo, const int FMF_FileVersion, unsigned short action, unsigned short option, size_t size_of_fms);
#endif
#ifndef FMF_EXCLUDE_LOCK
int lock_unlock (FMFILEINFO *finfo, unsigned short action, unsigned short option);
#endif
#ifndef	FMF_EXCLUDE_ARCHIVE
int archive_file (const char *filename, unsigned short *option);
#endif
//

//================
//THE FUNCTION
long int fmanage (FILE **f, const char *filename, const int FMF_FileVersion, const char extension[5], void *fms, size_t size_of_fms,
							 int posnumber, unsigned short action, unsigned short option)
{
	FILE *f_temp = NULL;

	FMFILEINFO finfo = {0, 0, 0, 0};
	void *fms_temp = NULL;

	int CopiedBlocks = 0;
	short int NoTempFileNeeded = 0;
	int DoNotOverwrite = 0;
	long int result = 0;
	int AddOk = 0;
	int EditSingleEntry = 0;
	int NewFileCreated = 0;

	void *kbq = kbd_queue ();
	unsigned short key;


	#ifdef FMF_DEBUG
		clrscr();
		FontSetSys(F_4x6);
		printf("action = %d\n"
						"option = %d\n"
						"-------------\n", action, option);
	#endif

	if ((result = check_excluded_functions(action, option)) != 0)
		return result;


	if ((action & FMF_NEW_FILE) != FMF_NEW_FILE)
		{

				#ifdef FMF_DEBUG
					printf("Load file informations\n");
				#endif

				//Return if no file is opened
				if (*f == NULL)
					return -5;

				//Move the file pointer to the beginning of the file
				if ((fseek(*f, 0, SEEK_SET)) != 0)
					return -1;

				//Read the version of fmanage
				if ((fread(&fmanageVersion, sizeof(fmanageVersion), 1, *f)) != 1)
					return -2;
				/*
				if ((option & FMF_NO_FINFO_CHECK) != FMF_NO_FINFO_CHECK)
					{
						//Check the version of fmanage and show an infodialog, if it's the wrong one
						if (fmanageVersion != FMANAGE_VERSION)
							{
								char str[200] = {0};
								sprintf(str, "Your file was made with version %d of fmanage!\n"
															"Your're using version %d of fmanage!\n"
															"Please contact the author of your programm or send an e-mail to the author of fmanage:\n"
															"Thomas Naeff - th.n@gmx.ch", fmanageVersion, FMANAGE_VERSION);
								DlgMessage("fmanage", str, BT_OK, BT_NONE);
								return -4;
							}
					}
				*/

				//Read the fileinfo at the beginning of the file
				if ((fread(&finfo, sizeof(FMFILEINFO), 1, *f)) != 1)
					return -2;

		}

	#ifndef	FMF_EXCLUDE_RETURN_FINFO
	if ((result = return_fileinfo(&finfo, action, option)) > -1)
		return result;
	#endif

	#ifndef FMF_EXCLUDE_LOCK
	if ((result = lock_unlock(&finfo, action, option)) < 1)
		return result;
	#endif

	#ifndef FMF_EXCLUDE_CHECK_FILEINFO
	if ((result = check_fileinfo(&finfo, FMF_FileVersion, action, option, size_of_fms)) < 0)
		return result;
	#endif





	//If you like to add a block somewhere in the file, but posnumber is not valid, return.
	#ifndef FMF_EXCLUDE_ADD_POS
	if ((action & FMF_ADD_POS) == FMF_ADD_POS)
		{
			if ((posnumber < 1) || (posnumber > (finfo.NumberOfBlocks + 1)))
				return -5;

			//If you like to add a block somewhere, and that's at the end
			if (posnumber == (finfo.NumberOfBlocks + 1))
				action = FMF_ADD;
		}
	#endif

	//If you like to remove a block from the file, but posnumber is not valid, return.
	//If you like to load a block from the file, but posnumber is not valid, return.
	#if (!defined FMF_EXCLUDE_REMOVE || !defined FMF_EXCLUDE_LOAD)
	if ((action & FMF_REMOVE) == FMF_REMOVE)
		{
			if ((posnumber < 1) || (posnumber > finfo.NumberOfBlocks))
				return -5;
		}
	#endif

	//If no block size is given (and check_fileinfo was swithed off with FMF_NO_FINFO_CHECK,
	//take the size of the blocks from the fileinfo
	if (size_of_fms <= 0)
		size_of_fms = finfo.SizeOfBlocks;

	//Load data?
	#ifndef	FMF_EXCLUDE_LOAD
	if ((action & FMF_LOAD) == FMF_LOAD)
		{
				#ifdef FMF_DEBUG
					printf("Load data [%d]\n", posnumber);
				#endif

							//Return if no file is opened
							if (*f == NULL)
								return -5;

							if (posnumber < 1)
								return -5;

							//Move the file pointer to the right position
							if ((fseek(*f, sizeof(fmanageVersion) + sizeof(FMFILEINFO) + ((posnumber - 1) * size_of_fms), SEEK_SET)) != 0)
								return -1;

							//Read one block from f to the structure
							if ((fread(fms, size_of_fms, 1, *f))  != 1)
								return -2;

							NoTempFileNeeded = 1;

		}
	#endif


	//Add data?
	//Edit data?
	//Remove data?
	//Copy all data?
	#if (!defined FMF_EXCLUDE_ADD || !defined FMF_ADD_POS || !defined FMF_EXCLUDE_EDIT)
	if (((action & FMF_NEW_FILE) != FMF_NEW_FILE)
		&& (NoTempFileNeeded == 0))
		{

				#ifdef FMF_DEBUG
					printf("Add/Edit/Remove %ld datablocks\n", finfo.NumberOfBlocks);
				#endif

							//Return if no file is opened
							if (*f == NULL)
								return -5;

							//Allocate memory for the temporary structure
							if ((fms_temp = malloc(size_of_fms)) == NULL)
								return -7;

							//Increase the number of stored blocks
							if (((action & FMF_ADD_POS) == FMF_ADD_POS)
									|| ((action & FMF_ADD) == FMF_ADD))
									{
										finfo.NumberOfBlocks = finfo.NumberOfBlocks + 1;
									}
							//Decrease the number of stored blocks
							if ((action & FMF_REMOVE) == FMF_REMOVE)
									finfo.NumberOfBlocks = finfo.NumberOfBlocks - 1;

							//Open the temporary file, where the content of f will be copyed to
							if ((f_temp = fopen(FMF_F_TEMP, "wb")) == NULL)
								return -1;

							//Write the version number of fmanage at the beginning of the file
							if ((fwrite(&fmanageVersion, sizeof(fmanageVersion), 1, f_temp))  != 1)
								{
									free(fms_temp);
									return -3;
								}


							//Write the fileinfo to the beginning of the file after fmanageVersion
							if ((fwrite(&finfo, sizeof(FMFILEINFO), 1, f_temp))  != 1)
								{
									free(fms_temp);
									return -3;
								}

							//Place the file pointer in f at the beginning of the first block (structure)
							if ((fseek(*f, sizeof(fmanageVersion) + sizeof(FMFILEINFO), SEEK_SET)) != 0)
								{
									free(fms_temp);
									return -1;
								}

							//Add data somewhere but when the file is empty, add it at the beginning.
							#ifndef FMF_EXCLUDE_ADD_POS
							if ((action & FMF_ADD_POS) == FMF_ADD_POS)
								{
											//<= 1 is used because NumberOfBlocks has already been increased.
											if (finfo.NumberOfBlocks <= 1)
												{

															#ifdef FMF_DEBUG
																printf("FMF_ADD_POS (the first block)\n");
															#endif
															//Write the structure to the beginning of the temporary file
															if ((fwrite(fms, size_of_fms, 1, f_temp))  != 1)
																{
																	free(fms_temp);
																	return -3;
																}

															AddOk = 1;
												}

								}
							#endif


							//Adjust the number of stored blocks when a block hast to be removed,
							//considering that NumberOfBlocks has already been increased and that
							//it will take one loop less, because of the "<" (used for the adjustment
							//whenn adding a block).
							if ((action & FMF_REMOVE) == FMF_REMOVE)
									finfo.NumberOfBlocks = finfo.NumberOfBlocks + 2;

							//Adjust the number of stored blocks when the first block hast to be edited
							//and only one block exists.
							//That's because of the "<" (used for the adjustment whenn adding a block).
							if ((action & FMF_EDIT) == FMF_EDIT)
									{
										if (finfo.NumberOfBlocks == 1)
											{
												finfo.NumberOfBlocks = 2;
												EditSingleEntry = 1;
											}
										else
											{
													finfo.NumberOfBlocks = finfo.NumberOfBlocks + 1;
											}

									}


							#ifdef FMF_DEBUG
								printf("Copy data [%ld] ", finfo.NumberOfBlocks);
							#endif

							//----- COPY -------
							//Copy all blocks from f to f_temp.
							//I use < and not <= because "finfo.NumberOfBlocks" has already been increased.
							for (CopiedBlocks = 1; CopiedBlocks < finfo.NumberOfBlocks; CopiedBlocks++)
								{

										#ifdef FMF_DEBUG
											printf("%d", CopiedBlocks);
										#endif

										//Read one block from f to the temporary structure fms_temp
										if ((fread(fms_temp, size_of_fms, 1, *f))  != 1)
											{
												free(fms_temp);
												return -2;
											}

										//Add data somewhere?
										#ifndef FMF_EXCLUDE_ADD_POS
										if (((action & FMF_ADD_POS) == FMF_ADD_POS)
												&& (AddOk == 0))
											{

														//Write the structure to the position in the temporary file given with posnumber
														if (CopiedBlocks == posnumber)
															{

																		#ifdef FMF_DEBUG
																			printf("-FMF_ADD_POS-");
																		#endif

																	//Write the structure to the position "posnumber" of the temporary file
																	if ((fwrite(fms, size_of_fms, 1, f_temp))  != 1)
																		{
																			free(fms_temp);
																			return -3;
																		}

																	AddOk = 1;

															}

											}
										#endif

										//Remove a block?
										#ifndef FMF_EXCLUDE_REMOVE
										if ((action & FMF_REMOVE) == FMF_REMOVE)
											{

														//Remove the block
														if (CopiedBlocks == posnumber)
															{

																		#ifdef FMF_DEBUG
																			printf("-FMF_REMOVE-");
																		#endif

																		continue;
															}

											}
										#endif

										//Edit data?
										#ifndef FMF_EXCLUDE_EDIT
										if ((action & FMF_EDIT) == FMF_EDIT)
											{

														//Edit the block
														if ((CopiedBlocks == posnumber)
																&& (fms != NULL))
															{
																#ifdef FMF_DEBUG
																	printf("-FMF_EDIT");
																#endif
																//Overwrite the readen data with the data given in the structure fms.
																if (CopiedBlocks == posnumber)
																	memcpy(fms_temp, fms, size_of_fms);
															}

											}
										#endif

										#ifdef FMF_DEBUG
											printf("-");
										#endif
										//Write the readen structure to the temporary file
										if ((fwrite(fms_temp, size_of_fms, 1, f_temp))  != 1)
											{
												free(fms_temp);
												return -3;
											}


										//If you edit a file with just one entry, in front of the loop NumberOfBlocks has been
										//set to 2, because otherwise the single entry would not be copied.
										//-> cancel the loop after copied the single entry.
										if (EditSingleEntry == 1)
											{
												finfo.NumberOfBlocks = 1;	//reset
												break;
											}

										//Exit copy when ESC was pressed
										if (!OSdequeue (&key, kbq))
											{
												if (key == KEY_ESC)
													{
														//Close and remove the temporary file
														fclose(f_temp);
														unlink(FMF_F_TEMP);
														return -1;
													}
											}

										//It would be nice to show the percent status of the copied blocks, but it makes the whole progress really slow!
										//sprintf(copyinfo, "Copy: %3.1f%%", (double)(100 / (double)finfo.NumberOfBlocks) * (double)CopiedBlocks);
										//ST_helpMsg(copyinfo);


								};
							//----- end of COPY -------

							#ifdef FMF_DEBUG
								printf(" >OK\n");
							#endif

							//Add data at the end?
							//Add data at "posnumber", but "posnumber" is at the end?
							#ifndef FMF_EXCLUDE_ADD_END
							if ((((action & FMF_ADD) == FMF_ADD)
									|| (posnumber > finfo.NumberOfBlocks))
									&& (AddOk == 0))
								{

											#ifdef FMF_DEBUG
												printf("FMF_ADD_END\n");
											#endif

											//Write the structure to the end of the temporary file
											if ((fwrite(fms, size_of_fms, 1, f_temp))  != 1)
												{
													free(fms_temp);
													return -3;
												}

											AddOk = 1;

								}
							#endif


			free(fms_temp);

		}
	#endif


	//New file?
	#ifndef	FMF_EXCLUDE_NEW_FILE
	if ((action & FMF_NEW_FILE) == FMF_NEW_FILE)
		{

				#ifdef FMF_DEBUG
					printf("FMF_NEW_FILE\n");
				#endif

							fclose(*f);

							//Try to open the file to manage
							//If it was possible to open it, ask if it should be overwritten.
							if ((*f = fopen(filename, "rb+")) != NULL)
								{
									//Automatically do not overwrite an existing file
									if ((option & FMF_NEW_NO_OVERWR) == FMF_NEW_NO_OVERWR)
										{
											DoNotOverwrite = 1;
										}
									else if ((option & FMF_NEW_OVERWR) == FMF_NEW_OVERWR)
										{
											DoNotOverwrite = 0; //Automatically overwrite an existing file
										}
									else
										{
											//Ask for overwriting an existing file
											if ((DlgMessage(filename, "Are you shure to overwrite your file?", BT_YES, BT_NO)) == KEY_ESC)
												DoNotOverwrite = 1;	//NO, do not overwrite
										}

									if (DoNotOverwrite == 0)
										{
											#ifdef FMF_DEBUG
												printf("Overwrite the old one\n");
											#endif

											//YES, overwrite it
											fclose(*f);
											unlink(filename);
										}
									else
										{
											#ifdef FMF_DEBUG
												printf("Do not overwrite the old one\n");
											#endif
										}

								}
							else
								{
									NewFileCreated = 1;
								}

							if (DoNotOverwrite == 0)
								{

									//Open a new file
									if ((*f = fopen(filename, "wb")) == NULL)
										return -1;

									//Set the default fileinfo
									fmanageVersion = FMANAGE_VERSION;

									finfo.Locked = FMF_LOCK_OFF;
									finfo.FileVersion = FMF_FileVersion;
									finfo.NumberOfBlocks = 0;
									finfo.SizeOfBlocks = size_of_fms;

									//Write the version number of fmanage at the beginning of the file
									if ((fwrite(&fmanageVersion, sizeof(fmanageVersion), 1, *f))  != 1)
										return -3;

									//Write the fileinfo structure to the file after fmanageVersion
									if ((fwrite(&finfo, sizeof(FMFILEINFO), 1, *f))  != 1)
										return -3;

									if ((option & FMF_NOEXT) != FMF_NOEXT)
										{

												#ifdef FMF_DEBUG
													printf("Add extension to f\n");
												#endif

												//Write the extension to the new file
												fputc (0, *f);
												fputs (extension, *f);
												fputc (0, *f);
												fputc (OTH_TAG, *f);

										}

								}

							//Do not allow to add a file extension to f_temp
							option |= FMF_NOEXT;

		}
	#endif


	//Add file extension?
	#ifndef	FMF_EXCLUDE_NOEXT
	if (((option & FMF_NOEXT) != FMF_NOEXT)
			&& (NoTempFileNeeded == 0))
		{

				#ifdef FMF_DEBUG
					printf("Add extension to f_temp\n");
				#endif

					//Write the extension to the file
					fputc (0, f_temp);
					fputs (extension, f_temp);
					fputc (0, f_temp);
					fputc (OTH_TAG, f_temp);

		}
	#endif


	if (((action & FMF_CLOSE) == FMF_CLOSE)
			|| (NoTempFileNeeded == 0))
		{

				#ifdef FMF_DEBUG
					printf("Close files\n");
				#endif

					fclose(*f);
					fclose(f_temp);

		}



	if (((action & FMF_NEW_FILE) != FMF_NEW_FILE)
			&& (NoTempFileNeeded == 0))
		{

				#ifdef FMF_DEBUG
					printf("Unlink and rename\n");
				#endif

					//Remove the old datafile and rename the temporary file as the datafile
					unlink(filename);
					rename(FMF_F_TEMP, filename);

		}



	#ifndef	FMF_EXCLUDE_ARCHIVE
		if ((result = archive_file(filename, &option)) < 0)
			return result;
	#endif


	//Should the file be closed or open at the end?
	if (((option & FMF_CLOSE) != FMF_CLOSE)
			&& (NoTempFileNeeded == 0))
		{

				#ifdef FMF_DEBUG
					printf("Reopen\n");
				#endif

					//Open the file to manage again
					if ((*f = fopen(filename, "rb+")) == NULL)
						return -1;
		}

	#ifdef FMF_DEBUG
		printf("End of fmanage\n"
						"Press any key to continue...\n");
		ngetchx();
	#endif


	if (NewFileCreated == 1)
		return 1;

	if (DoNotOverwrite == 1)
		return 0;
	else
		return finfo.NumberOfBlocks; 	//Return the number of blocks (structures) stored in the file

}
//================

//==========================================================================================================
//==========================================================================================================

int check_excluded_functions (unsigned short action, unsigned short option)
{
	int FoundExcludedFunction = 0;

	//---------------------------------------------------------------
	//Check for excluded but still used parts of the function
	//(check up fmanage_user.h for more information)

	#ifdef FMF_DEBUG
		printf("Check if a used function is excluded\n");
	#endif

	//action
	#ifdef	FMF_EXCLUDE_RETURN_FINFO
		if ((action & FMF_RETURN_FINFO) == FMF_RETURN_FINFO)
			{
				#ifdef FMF_DEBUG
					printf("FMF_EXCLUDE_RETURN_FINFO is defined!\n");
				#endif

				FoundExcludedFunction = 1;
			}
	#endif

	#ifdef	FMF_EXCLUDE_LOAD
		if ((action & FMF_LOAD) == FMF_LOAD)
			{
				#ifdef FMF_DEBUG
					printf("FMF_EXCLUDE_LOAD is defined!\n");
				#endif

				FoundExcludedFunction = 1;
			}
	#endif

	#ifdef	FMF_EXCLUDE_ADD_POS
		if ((action & FMF_ADD_POS) == FMF_ADD_POS)
			{
				#ifdef FMF_DEBUG
					printf("FMF_EXCLUDE_ADD_POS is defined!\n");
				#endif

				FoundExcludedFunction = 1;
			}
	#endif

	#ifdef	FMF_EXCLUDE_ADD
		if ((action & FMF_ADD) == FMF_ADD)
			{
				#ifdef FMF_DEBUG
					printf("FMF_EXCLUDE_ADD is defined!\n");
				#endif

				FoundExcludedFunction = 1;
			}
	#endif


	#ifdef	FMF_EXCLUDE_EDIT
		if ((action & FMF_EDIT) == FMF_EDIT)
			{
				#ifdef FMF_DEBUG
					printf("FMF_EXCLUDE_EDIT is defined!\n");
				#endif

				FoundExcludedFunction = 1;
			}
	#endif

	#ifdef	FMF_EXCLUDE_NEW_FILE
		if ((action & FMF_NEW_FILE) == FMF_NEW_FILE)
			{
				#ifdef FMF_DEBUG
					printf("FMF_EXCLUDE_NEW_FILE is defined!\n");
				#endif

				FoundExcludedFunction = 1;
			}
	#endif

	//option
	#ifdef	FMF_EXCLUDE_NOEXT
		if ((option & FMF_NOEXT) == FMF_NOEXT)
			{
				#ifdef FMF_DEBUG
					printf("FMF_EXCLUDE_NOEXT is defined!\n");
				#endif

				FoundExcludedFunction = 1;
			}
	#endif

	#ifdef	FMF_EXCLUDE_ARCHIVE
		if ((option & FMF_ARCHIVE) == FMF_ARCHIVE)
			{
				#ifdef FMF_DEBUG
					printf("FMF_EXCLUDE_ARCHIVE is defined!\n");
				#endif

				FoundExcludedFunction = 1;
			}
	#endif

	#ifdef	FMF_EXCLUDE_LOCK
		if (((option & FMF_LOCK_FILE) == FMF_LOCK_FILE)
				|| ((option & FMF_UNLOCK_FILE) == FMF_UNLOCK_FILE))
			{
				#ifdef FMF_DEBUG
					printf("FMF_EXCLUDE_LOCK is defined!\n");
				#endif

				FoundExcludedFunction = 1;
			}
	#endif
	//---------------------------------------------------------------

	if (FoundExcludedFunction == 1)
		{
			#ifdef FMF_DEBUG
				printf("Press any key to continue...\n");
				ngetchx();
			#endif
			return -6;
		}
	else
		return 0;

}

//==========================================================================================================
#ifndef	FMF_EXCLUDE_RETURN_FINFO
long int return_fileinfo (FMFILEINFO *finfo, unsigned short action, unsigned short option)
{

	//Return fileinfo?

	if ((action & FMF_RETURN_FINFO) == FMF_RETURN_FINFO)
		{

				#ifdef FMF_DEBUG
					printf("Return fileinfo\n");
				#endif

				if ((option & FMF_RETURN_FMV) == FMF_RETURN_FMV)
					{
						#ifdef FMF_DEBUG
							printf("-> fmanageVersion = %d\n", fmanageVersion);
						#endif
						return fmanageVersion;
					}

				//Just return the fileinfo
				//No other options than FMF_RETURN_xyz can be used

				if ((option & FMF_RETURN_FV) == FMF_RETURN_FV)
					{
						#ifdef FMF_DEBUG
							printf("-> FileVersion = %d\n", finfo->FileVersion);
						#endif
						return finfo->FileVersion;
					}


				if ((option & FMF_RETURN_NOB) == FMF_RETURN_NOB)
					{
						#ifdef FMF_DEBUG
							printf("-> NumberOfBlocks = %ld\n", finfo->NumberOfBlocks);
						#endif
						return finfo->NumberOfBlocks;
					}

				if ((option & FMF_RETURN_SOB) == FMF_RETURN_SOB)
					{
						#ifdef FMF_DEBUG
							printf("-> SizeOfBlocks = %d\n", finfo->SizeOfBlocks);
						#endif
						return finfo->SizeOfBlocks;
					}

				if ((option & FMF_RETURN_L) == FMF_RETURN_L)
					{
						#ifdef FMF_DEBUG
							printf("-> Locked = %d\n", finfo->Locked);
						#endif
						return finfo->Locked;
					}

			}

	return -1;

}
#endif
//==========================================================================================================
#ifndef FMF_EXCLUDE_CHECK_FILEINFO
int check_fileinfo (FMFILEINFO *finfo, const int FMF_FileVersion, unsigned short action, unsigned short option, size_t size_of_fms)
{

	if ((option & FMF_NO_FINFO_CHECK) == FMF_NO_FINFO_CHECK)
		return 0;

	//Check the file informations
	if ((action & FMF_NEW_FILE) != FMF_NEW_FILE)
		{
				//Test if the stored blocks have the right size
				if (finfo->SizeOfBlocks != size_of_fms)
					return -4;

				//Test if the version of the file is the right one
				if (finfo->FileVersion != FMF_FileVersion)
					return -4;
		}


	return 0;
}
#endif
//==========================================================================================================
#ifndef FMF_EXCLUDE_LOCK
int lock_unlock (FMFILEINFO *finfo, unsigned short action, unsigned short option)
{
	if ((action & FMF_LOAD) == FMF_LOAD)
		return 1;

		//Unlock file?
		if ((option & FMF_UNLOCK_FILE) == FMF_UNLOCK_FILE)
			{

					#ifdef FMF_DEBUG
						printf("Unlock file\n");
					#endif

					finfo->Locked = FMF_LOCK_OFF;

			}


		//Locked or not?
		if (finfo->Locked == FMF_LOCK_ON)
			{
					#ifdef FMF_DEBUG
						printf("File is locked!\n");
					#endif

					//DlgMessage("fmanage", "File is locked!", BT_OK, BT_NONE);
					return -10;

			}
		else
			{
					#ifdef FMF_DEBUG
						printf("File is not locked.\n");
					#endif
			}


		//Lock file?
		if ((option & FMF_LOCK_FILE) == FMF_LOCK_FILE)
			{

					#ifdef FMF_DEBUG
						printf("Lock file\n");
					#endif

					finfo->Locked = FMF_LOCK_ON;

			}


	return 1;
}
#endif
//==========================================================================================================
#ifndef	FMF_EXCLUDE_ARCHIVE
int archive_file (const char *filename, unsigned short *option)
{
	//Archive file?
	if ((*option & FMF_ARCHIVE) == FMF_ARCHIVE)
		{

				#ifdef FMF_DEBUG
					printf("Archive\n");
				#endif

			if ((EM_moveSymToExtMem (SYMSTR(filename), HS_NULL)) == FALSE)
				return -1;

			//Do not open the file again
			*option |= FMF_CLOSE;
		}

	return 0;
}
#endif
//==========================================================================================================

//==========================================================================================================

//==========================================================================================================

//==========================================================================================================