// Header File
// Created 11.02.2006; 16:30:48
/*--------------------------------------------------------------\
| 																															|
| 						Header file of the function fmanage								|
|																																|
|										by: Thomas Naeff														|
|												www.mytinyarchive.ch.vu									|
|												th.n@gmx.ch															|
|																																|
|																																|
|						Do not change anything in this file!								|
|																																|
\*-------------------------------------------------------------*/

 /********************************************************************************
 ********************************************************************************
 * - the header file of the function fmanage from fmanage.a -
 *
 * Programm name:		---
 * File name:				fmanage.h
 *
 * Short Description: Function to "manage" a file. This function can be used to
 *										create a new file, add/edit/remove data, lock the file,
 *										add a specific file extension, archive the file, ...
 *
 * Description:
 *	-
 *
 ********************************************************************************
 *
 *	History
 * =========
 *
 * Anytime before April 2006:
 * ---------------------------
 * The first version of fmanage
 *
 *
 ********************************************************************************
 *********************************************************************************/

#ifndef _FMANAGE_H_	//If the header file has not been included yet...

#define	_FMANAGE_H_

//The version of the function fmanage. This number will be stored at the beginning on every file.
#define	FMANAGE_VERSION		1


//DEFINITIONS

//action: (they SHOULDN'T be OR-ed!)
#define		FMF_NEW_FILE			1			//Write the needed filestructure to the file
#define		FMF_ADD						2			//Add the new structure at the end of the file (fmanage returns NumberOfBlocks for success and -1 if not successful)
#define		FMF_ADD_POS				4			//Add the new structure after the position, given with posnumber (fmanage returns 0 for success and -1 if not successful)
#define		FMF_SEARCH				8			//Search for the structure (fmanage returns the position number of the structure in the file or 0 if the data has not been found)
#define		FMF_LOAD					16		//Load the structure with the position number given in posnumber
#define		FMF_REMOVE				32		//Remove the structure with the position number given in posnumber (fmanage returns NumberOfBlocks for success and -1 if not successful)
#define		FMF_EDIT					64		//Replace the structure with the positon number given in posnumber (fmanage returns NumberOfBlocks for success and -1 if not successful)
#define		FMF_RETURN_FINFO	128		//Returns the fileinfo OR-ed with FMF_RETURN_FINFO


//option: These definitions can be used when FMF_RETURN_FINFO is used in action
#define		FMF_RETURN_FV				1			//Returns the file version
#define		FMF_RETURN_NOB			2			//Returns the number of blocks (structures) stored in the file
#define		FMF_RETURN_SOB			4			//Returns the size of the blocks (structures) stored in the file
#define		FMF_RETURN_L				8			//Returns if the file is locked (FMF_LOCK_ON) or not (FMF_LOCK_OFF)

#define		FMF_RETURN_FMV			128		//Returns the version number of fmanage the file was created with.



//option: (they can be OR-ed)
#define		FMF_ARCHIVE					1			//Close the file and archive it at the end
#define		FMF_NOEXT						2			//Don't add the file extension to the file
#define		FMF_CLOSE						4			//Close the file at the end
#define		FMF_NEW_OVERWR			8			//If "action" is FMF_NEW_FILE, add this option so an existing file will be overwritten without asking
#define		FMF_NEW_NO_OVERWR 	16		//If "action" is FMF_NEW_FILE, add this option so an existing file will NOT be overwritten without asking
#define		FMF_LOCK_FILE				32		//Lock the file (use together with FMF_EDIT)
#define		FMF_UNLOCK_FILE			64		//Unlock the file (use together with FMF_EDIT)
#define		FMF_NO_FINFO_CHECK	128		//Don't check the file informations (block size and file version)



//Filenames
#define		FMF_F_TEMP		"main\\f_temp"


int	fmanageVersion;		//The version of the function fmanage. The version number will be stored at the beginning of each file.
											//I made this, because if I find a bug in the function and if I have to change important things, this number
											//will be increased.


//The structure with file informations
//This structure will be adden to each file, right after fmanageVersion
typedef struct
	{

		int Locked;											// 1 = You can't edit the file. 0 = you can do whatever you want to.
		int FileVersion;								//The version of the file. To determine the file structure, if you have more than one programmversion
		long int NumberOfBlocks;				//The number of blocks (structures) stored in the file.
		unsigned int SizeOfBlocks;			//The size of the blocks (structures) stored in the file.

	} FMFILEINFO;


enum lock_file {FMF_LOCK_OFF, FMF_LOCK_ON};


//PROTOTYPES
long int fmanage (FILE **f, const char *filename, const int FMF_FileVersion, const char extension[5], void *fms, size_t size_of_fms,
							 int posnumber, unsigned short action, unsigned short option);
//f									A pointer to the file pointer of your file
//filename					The name of the file
//FMF_FileVersion		The version of your file (set it to 0 if you don't care about it)
//extension					The file extension as string (max. 4 characters)
//fms								The pointer to the structure with the data (Memory has to be allocated for the structure!)
//size_of_fms				The size of your structure (sizeof(YOURSTRUCT))
//posnumber					The position number of the data. Used for FMF_EDIT, FMF_REMOVE or FMF_ADD_POS
//action						How to process with the structure
//option						Additional options (they can be ORed)





/*================================================================================================

Use the following definitions to manipulate the sourcecode.
After each change, you have to compile the sourcecode again!

I give no warranty that fmanage will work correct if you use these things, but it should ;)

================================================================================================*/

/*--------------------------------------------------------------
| Use these definitions, to disable unused parts of the function
| and make your programm a little bit smaller.
| Take care that you don't use a part which is excluded!!!
| fmanage will check if you use a disabled function and return -6.
\*-------------------------------------------------------------*/

//#define		FMF_EXCLUDE_RETURN_FINFO			//You can't use the action FMF_RETURN_FINFO
//#define		FMF_EXCLUDE_LOAD							//You can't use the action FMF_LOAD
//#define		FMF_EXCLUDE_ADD_POS						//You can't use the action FMF_ADD_POS
//#define		FMF_EXCLUDE_ADD								//You can't use the action FMF_ADD
//#define		FMF_EXCLUDE_EDIT							//You can't use the action FMF_EDIT
//#define		FMF_EXCLUDE_REMOVE						//You can't use the action FMF_REMOVE
//...
//#define		FMF_EXCLUDE_NEW_FILE					//You can't use the action FMF_NEW_FILE

//#define		FMF_EXCLUDE_NOEXT							//You can't use the option FMF_NOEXT
//#define		FMF_EXCLUDE_ARCHIVE						//You can't use the option FMF_NOARCHIVE
//...
//#define		FMF_EXCLUDE_LOCK							//You can't use the option FMF_LOCK_FILE and FMF_UNLOCK_FILE. The file will also not be tested if it is locked or not.

//#define		FMF_EXCLUDE_CHECK_FILEINFO		//It will not check the file informations (Valid file version, right size of blocks, ...)



/*--------------------------------------------------------------
| With FMF_DEBUG defined, when the function is called, the display
| will be cleared with clrscr(); and some informations in 4x6-Font
| will be showed.
\*--------------------------------------------------------------*/
//#define		FMF_DEBUG


#endif //of ifndef _FMANAGE_H_


