// C Source File
// Created 18.02.2006; 18:07:27

 /********************************************************************************
 ********************************************************************************
 * Programm name:		thedb - The handy education database
 * File name:				create_new_file.c
 * Function name:		create_new_file
 * Header file:			---
 * Other functions:
 *	- print_str_in_window
 *	- swap_active_list_string
 *	- switch_active_form_and_list_and_read
 *	- switch_k_right_left
 *	- list_process_paste
 *
 * Short Description: List the data ot the active file in a window
 *										With several functions to scroll, search, edit, remove, solve, ...
 *
 * Description:
 *	-
 *
 ********************************************************************************
 *********************************************************************************/

#include <tigcclib.h>
#include "fmanage.h"

#include "structures.h"
#include "global.h"
#include "calc.h"

//
void print_str_in_window (WINDOW *ListWindow, const char *str, unsigned int strnumber, int result);
void swap_active_list_string (CONFSTRUCT *confstruct, int CursorPos, int action, int *print_mode);
void switch_active_form_and_list_and_read (FILE *datafile, WINDOW *ListWindow, CONFSTRUCT *confstruct,
																						FORMULADATA *formuladata, TEXTDATA *textdata, int result, int lines);
int switch_k_right_left (CONFSTRUCT *confstruct, int gk, int CursorPos, int *print_mode);
void list_process_paste (CONFSTRUCT *confstruct, FILE **file, int *Marked, int MarkedAction, int MoveDataTo);
//

//==============================================================================
//List the data in a window
int list_data (CONFSTRUCT *confstruct)
{
	WINDOW *ListWindow;
	WIN_RECT ListWinr = {1, 1, LCD_WIDTH - 1, LCD_HEIGHT - 9};

	FILE *datafile = NULL;

	FORMULADATA formuladata;
	TEXTDATA textdata;

	int NumOfBlocks = 0;
	int lines = 0;
	int start = 1;
	int result = 0;
	int gk = 0;
	int RefreshList = 1;
	int Pos = 1;
	int Marked = 0;
	int MarkedAction = 0;
	int print_mode = 0;
	int datanr = 0;

	GrayOff();
	clrscr();

	if ((check_file_validity(confstruct, ActivePath, 1)) == 0)
		return -1;

	if ((datafile = fopen(ActivePath, "rb")) == NULL)
		{
			DlgMessage("thedb[6]", "Failed to open!", BT_OK, BT_NONE);
			return -1;
		}

	//Load the number of blocks
	NumOfBlocks = fmanage(&datafile, NULL, 0, NULL, NULL, 0, 0, FMF_RETURN_FINFO, FMF_RETURN_NOB);

	if (NumOfBlocks <= 0)
		{
			DlgMessage("thedb[7]", "File is empty!", BT_OK, BT_NONE);
			fclose(datafile);
			return -1;
		}

	//Reload the position of the cursor block.
	//Reload the first showed entry.
	switch (confstruct->ActiveForm)
		{
			case FORM_FORMULA:
					if ((confstruct->PosFormula[confstruct->ActivePathFormula] > 0) && (confstruct->PosFormula[confstruct->ActivePathFormula] <= MAX_DISP_LINES))
						Pos = confstruct->PosFormula[confstruct->ActivePathFormula];
					if ((confstruct->StartFormula[confstruct->ActivePathFormula] > 0) && (confstruct->StartFormula[confstruct->ActivePathFormula] <= NumOfBlocks - MAX_DISP_LINES + 1))
						start = confstruct->StartFormula[confstruct->ActivePathFormula];
				break;
			case FORM_TEXT:
					if ((confstruct->PosText[confstruct->ActivePathText] > 0) && (confstruct->PosText[confstruct->ActivePathText] <= MAX_DISP_LINES))
						Pos = confstruct->PosText[confstruct->ActivePathText];
					if ((confstruct->StartText[confstruct->ActivePathText] > 0) && (confstruct->StartText[confstruct->ActivePathText] <= NumOfBlocks - MAX_DISP_LINES))
						start = confstruct->StartText[confstruct->ActivePathText];
				break;
			default:
				break;
		}

	if (Pos > NumOfBlocks)
		Pos = 1;

	//Draw ListWindow
	ListWindow = HeapAllocPtr (sizeof (WINDOW));
  WinOpen(ListWindow, &ListWinr, WF_TTY | WF_SAVE_SCR);
  WinActivate (ListWindow);
  WinFont(ListWindow, F_4x6);

	GrayOn();


	while (1)
		{

			//Clear the screen and show the whole list again
			if (RefreshList == 1)
				{
					GrayOn();
					WinClr(ListWindow);
					//Load the number of blocks. In this way, the file pointer will also be at the right position.
					NumOfBlocks = fmanage(&datafile, NULL, 0, NULL, NULL, 0, 0, FMF_RETURN_FINFO, FMF_RETURN_NOB);
					switch (confstruct->ActiveForm)
						{
							case FORM_FORMULA:
									fseek(datafile, (start - 1) * sizeof(FORMULADATA), SEEK_CUR);
								break;
							case FORM_TEXT:
									fseek(datafile, (start - 1) * sizeof(TEXTDATA), SEEK_CUR);
								break;
							default:
								break;
						}

					for (lines = start; (lines <= ((start - 1) + MAX_DISP_LINES)) && (lines <= NumOfBlocks); lines++)
						switch_active_form_and_list_and_read (datafile, ListWindow, confstruct, &formuladata, &textdata, result, lines);

					lines--;
				}

			//Set the title
			swap_active_list_string(confstruct, Pos, 1, &print_mode);
			//Set the cursor bar
			WinFill (ListWindow, &(WIN_RECT){1, (Pos - 1) * 6 + 1, LCD_WIDTH - 7, ((Pos - 1) * 6) + 6}, A_XOR);
			//If CUT or COPY is choosen
			if ((MarkedAction != 0) && ((start <= Marked) || ((start + MAX_DISP_LINES) >= Marked)))
				WinFill (ListWindow, &(WIN_RECT){2, (Marked - start) * 6, LCD_WIDTH - 8, (Marked - start) * 6 + 5}, A_XOR);

			//Wait for a keypress
			gk = ngetchx();

			//Remove the old cursor bar
			WinFill (ListWindow, &(WIN_RECT){1, (Pos - 1) * 6 + 1, LCD_WIDTH - 7, ((Pos - 1) * 6) + 6}, A_XOR);
			//If CUT or COPY is choosen
			if ((MarkedAction != 0) && ((start <= Marked) || ((start + MAX_DISP_LINES) >= Marked)))
				WinFill (ListWindow, &(WIN_RECT){2, (Marked - start) * 6, LCD_WIDTH - 8, (Marked - start) * 6 + 5}, A_XOR);

			RefreshList = 0;
			if (MarkedAction == 0)
				ST_eraseHelp();
			else
				ST_helpMsg("Choose a new location and press PASTE");

			//---------------------------------------------
			switch (gk)
				{
					case KEY_2ND + KEY_DOWN:
					case KEY_DOWN:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
						Pos++;
							if ((Pos > MAX_DISP_LINES) || (Pos > NumOfBlocks))	//If the cursor bar has reached the bottom
								{
									Pos = MAX_DISP_LINES;
									if (gk == KEY_2ND + KEY_DOWN)
										{
											start += 4;
											RefreshList = 1;
										}
									start++;

									if ((start + MAX_DISP_LINES) > NumOfBlocks + 1)		//If the end of the data has been reached
										{											//Jump to the beginning
											RefreshList = 1;		//Refresh the list
											start = 1;					//Start with the first data
											Pos = 1;						//Set the cursor block to the top
											break;
										}

									lines++;
									switch_active_form_and_list_and_read(datafile, ListWindow, confstruct, &formuladata, &textdata, result, lines);
									break;
								}

							if (gk == KEY_2ND + KEY_DOWN)		//If 2nd+Down has been pressed, first move the cursor block to the bottom. If the cursor block was already
								{
									if (NumOfBlocks > MAX_DISP_LINES)
										Pos = MAX_DISP_LINES;					//at the bottom, the list has already been moved.
									else
										Pos = NumOfBlocks;
								}

						break;
					case KEY_2ND + KEY_UP:
					case KEY_UP:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
						Pos--;
							if (Pos < 1)							//If the cursor bar has reached the top
								{
									Pos = 1;
									if (gk == KEY_2ND + KEY_UP)
										start -= 4;
									start--;

									if (start < 1)			//If the data at the top one is the first one
										{																							//Jump to the end
											RefreshList = 1;														//Refresh the list
											start = NumOfBlocks - MAX_DISP_LINES + 1;		//Show the last MAX_DISP_LINES data
											if (start < 1)
												start = 1;
											Pos = MAX_DISP_LINES;												//Set the cursor block to the bottom
												if (Pos > NumOfBlocks)
													Pos = NumOfBlocks;
											break;
										}

									RefreshList = 1;			//refresh the list
									break;
								}

							if (gk == KEY_2ND + KEY_UP)		//If 2nd+Up has been pressed, first move the cursor block to the top. If the cursor block was already
								Pos = 1;										//at the top, the list has already been moved.
						break;
					case KEY_RIGHT:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
					case KEY_LEFT:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							RefreshList = switch_k_right_left(confstruct, gk, Pos, &print_mode);
						break;
					case KEY_HOME:
							Pos = 1;
							start = 1;
							RefreshList = 1;
						break;
					case KEY_BACKSPACE:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							if (check_if_in_use_bit_set(0) == -1)
								{
									RefreshList = 2;
									break;
								}
							switch (confstruct->ActiveForm)
								{
									case FORM_TEXT:
											new_entry_text(confstruct, FMF_EDIT, start + Pos - 1);
										break;
									case FORM_FORMULA:
											new_entry_formula(confstruct, FMF_EDIT, start + Pos - 1);
										break;
									default:
										break;
								}
							//restart list_data
							RefreshList = 2;
						break;
					case KEY_CLEAR:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							if (check_if_in_use_bit_set(0) == -1)
								{
									RefreshList = 2;
									break;
								}
							switch (confstruct->ActiveForm)
								{
									case FORM_TEXT:
											remove_entry(start + Pos - 1, ActivePath, FMF_TEXT_FILE_VERSION, FILE_TEXT_EXTENSION);
										break;
									case FORM_FORMULA:
											remove_entry(start + Pos - 1, ActivePath, FMF_FORMULA_FILE_VERSION, FILE_FORMULA_EXTENSION);
										break;
									default:
										break;
								}
							RefreshList = 2;
						break;
					case KEY_ENTER:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							if (check_if_in_use_bit_set(0) == -1)
								{
									RefreshList = 2;
									break;
								}
								copy_to_clipboard(confstruct, start + Pos - 1, NULL);
								RefreshList = 1;
						break;
					case KEY_ON:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							off();
						break;
					case K_COPY:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							if ((check_if_file_is_locked(datafile)) == -1)
								RefreshList = 2;
							Marked = start + Pos - 1;
							MarkedAction = MA_COPY;
						break;
					case K_CUT:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							if ((check_if_file_is_locked(datafile)) == -1)
								RefreshList = 2;
							Marked = start + Pos - 1;
							MarkedAction = MA_CUT;
						break;
					case K_PASTE:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							list_process_paste(confstruct, &datafile, &Marked, MarkedAction, start + Pos - 1);
							Marked = 0;
							MarkedAction = 0;
							RefreshList = 2;
						break;
					case KEY_APPS:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							if (check_if_in_use_bit_set(0) == -1)
								{
									RefreshList = 2;
									break;
								}
							int result = 1;
							datanr = start + Pos - 1;
							while ((result = view2d(confstruct, &datafile, NULL, ListWindow, &ListWinr, &datanr, &print_mode)) != 0)
								{
									//Reopen the file for reloading
									datafile = freopen(ActivePath, "rb", datafile);
									//Break the loop if F1 or F6 has been pressed
									if ((result == KEY_F1) || (result == KEY_F6) || (result == KEY_DIAMOND + KEY_F1))
										break;
								};
							//Do not exit the switch statement when F1 or F6 has been pressed but continue to process_vars
							if ((result != KEY_F1) && (result != KEY_F6) && (result != KEY_DIAMOND + KEY_F1))
								{
									RefreshList = 2;
									break;
								}
							else
								{
									gk = result;
								}
					case KEY_F1: 	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
					case KEY_F6:
					case KEY_DIAMOND + KEY_F1:
							if (check_if_in_use_bit_set(0) == -1)
								{
									RefreshList = 2;
									break;
								}

							if (datanr == 0)	//Set datanr, if it's not already done at "case KEY_APPS"
								datanr = start + Pos - 1;

							if (confstruct->ActiveForm == FORM_FORMULA)
								{
									while((process_vars(confstruct, ListWindow, &ListWinr, datanr, &print_mode, &gk)) == 1);
									//Reopen the file for reloading
									datafile = freopen(ActivePath, "rb", datafile);
								}
						datanr = 0;
						RefreshList = 2;
						break;
					case KEY_F3:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							Pos -= 1;
							result = search_entry(&datafile, confstruct, &start, &Pos, 0);
							if (result == -1)
								Pos += 1;
							RefreshList = 1;
						break;
					case KEY_F5: 	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							RefreshList = 2;
						break;
					case KEY_F8: 	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							//2nd + F3
							//Continue searching
							result = search_entry(&datafile, confstruct, &start, &Pos, 1);
							RefreshList = 1;
						break;
					default:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
						break;
				}
			//---------------------------------------------

			//Exit the Programm, if ESC was pressed
			//If COPY or CUT is still active, deactivate it first
			if (gk == KEY_ESC)
				{
					if ((MarkedAction != 0) || (Marked != 0))
						{
							Marked = 0;
							MarkedAction = 0;
						}
					else
						{
							break;
						}


					ST_eraseHelp();
				}

			//Save the selected entry.
			//Save the first showed entry.
			switch (confstruct->ActiveForm)
				{
					case FORM_TEXT:
							confstruct->PosText[confstruct->ActivePathText] = Pos;
							confstruct->StartText[confstruct->ActivePathText] = start;
						break;
					case FORM_FORMULA:
					default:
						confstruct->PosFormula[confstruct->ActivePathFormula] = Pos;
						confstruct->StartFormula[confstruct->ActivePathFormula] = start;
					break;
				}

			if (RefreshList == 2)
				{
					//restart list_data
					swap_active_list_string(confstruct, Pos, 1, &print_mode);
				  WinClose (ListWindow);
				  HeapFreePtr (ListWindow);
				  fclose(datafile);
				  return 1;
				  break;
				}

		};


	swap_active_list_string(confstruct, Pos, 0, &print_mode);
	//Close all windows
  WinClose (ListWindow);
  HeapFreePtr (ListWindow);
  fclose(datafile);
	return 0;
}




//===============================================================================================
//Print a new string in the window
//If the window is full, the top line will disappear
void print_str_in_window (WINDOW *ListWindow, const char *str, unsigned int strnumber, int result)
{
	char fstr[WinWidth(ListWindow) / 2];
	char number[5] = {0};
	int chars = 0;

	sprintf(number, "%2d: ", strnumber);
	WinStr(ListWindow, number);


	//Print out an error or the formula
	if (result != 1)
		strcpy(fstr, "! Failed to load !");
	else
		{
			//Look how many characters of the current string can be printed out in the window
			if ((chars_in_window_on_one_line(WinWidth(ListWindow) - 30, str, &chars)) == 0)
				{
					strncpy(fstr, str, chars);
					//Cut off
					fstr[chars] = '\0';
					//Add the "..."
					strcat(fstr, " ...");
				}
			else
				{
					strcpy(fstr, str);
				}

		}

	//Print the string
	WinStr(ListWindow, fstr);
	//Add a newline
	WinStr(ListWindow, "\n");

	return;
}

//===============================================================================================
//Swap the title of the active list view (the big gray font in the top right corner)
void swap_active_list_string (CONFSTRUCT *confstruct, int CursorPos, int action, int *print_mode)
{
	int y = 5;
	int yAdj = 0;
	char str[20] = {0};

	FontSetSys(F_8x10);
	GraySetAMSPlane (LIGHT_PLANE);

	//Move the title away from the cursor
	if (CursorPos <= (MAX_DISP_LINES / 2))
		y = LCD_HEIGHT - 21;


	switch (confstruct->ActiveForm)
		{
			case FORM_FORMULA:
					switch (confstruct->ActiveList)
						{
							case FORMULA:
									yAdj = 70;
									strcpy(str, "Formula");
									*print_mode = 1;
								break;
							case DESC:
									yAdj = 90;
									strcpy(str, "Description");
									*print_mode = 0;
								break;
							default:	//NAME
									confstruct->ActiveList = NAME;;
									yAdj = 60;
									strcpy(str, "Name");
									*print_mode = 0;
								break;
						}
				break;
			case FORM_TEXT:
					switch (confstruct->ActiveList)
						{
							case TEXT:
									yAdj = 55;
									strcpy(str, "Text");
									*print_mode = 0;
								break;
							default:	//TITLE
									confstruct->ActiveList = TITLE;
									yAdj = 60;
									strcpy(str, "Title");
									*print_mode = 0;
								break;
						}
				break;
			default:
				break;
		}

		//Clear the old titles on the top and bottom
		DrawStr(LCD_WIDTH - 110, 5, "              ", A_REPLACE);
		DrawStr(LCD_WIDTH - 110, LCD_HEIGHT - 21, "              ", A_REPLACE);

		if (action != 0)
			{
				//Draw the new title
				DrawStr(LCD_WIDTH - yAdj, y, str, A_NORMAL);
			}



		FontSetSys(F_4x6);
		GraySetAMSPlane (DARK_PLANE);

	return;

}

//===============================================================================================
//Reads the data from the right file and shows the right entry from the structure
void switch_active_form_and_list_and_read (FILE *datafile, WINDOW *ListWindow, CONFSTRUCT *confstruct,
																						FORMULADATA *formuladata, TEXTDATA *textdata, int result, int lines)
{

	switch (confstruct->ActiveForm)
		{
			case FORM_FORMULA:
					switch (confstruct->ActiveList)
						{
							case FORMULA:
									result = fread(formuladata, sizeof(FORMULADATA), 1, datafile);
									print_str_in_window(ListWindow, formuladata->formula, lines, result);
								break;
							case DESC:
									result = fread(formuladata, sizeof(FORMULADATA), 1, datafile);
									print_str_in_window(ListWindow, formuladata->desc, lines, result);
								break;
							default:	//NAME
									result = fread(formuladata, sizeof(FORMULADATA), 1, datafile);
									print_str_in_window(ListWindow, formuladata->name, lines, result);
								break;
						}
				break;
			case FORM_TEXT:
					switch (confstruct->ActiveList)
						{
							case TEXT:
									result = fread(textdata, sizeof(TEXTDATA), 1, datafile);
									print_str_in_window(ListWindow, textdata->text, lines, result);
								break;
							default:	//TITLE
									confstruct->ActiveList = TITLE;
									result = fread(textdata, sizeof(TEXTDATA), 1, datafile);
									print_str_in_window(ListWindow, textdata->title, lines, result);
								break;
						}
				break;
			default:
				break;
		}


	return;
}

//===============================================================================================
//Changes the view when K_LEFT or K_RIGHT has been pressed
int switch_k_right_left (CONFSTRUCT *confstruct, int gk, int CursorPos, int *print_mode)
{

	switch (confstruct->ActiveForm)
		{
			case FORM_FORMULA:
					switch (gk)
						{
							case KEY_RIGHT:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
									if ((confstruct->ActiveList + 1) > DESC)
										{
											confstruct->ActiveList = NAME;
											break;
										}
									confstruct->ActiveList++;
								break;
							case KEY_LEFT:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
									if ((confstruct->ActiveList - 1) < NAME)
										{
											confstruct->ActiveList = DESC;
											break;
										}
									confstruct->ActiveList--;
								break;
							default:
								break;
						}
					swap_active_list_string(confstruct, CursorPos, 1, print_mode);
					return 1;
				break;
			case FORM_TEXT:
					switch (gk)
						{
							case KEY_RIGHT:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
									if ((confstruct->ActiveList + 1) > TEXT)
										{
											confstruct->ActiveList = TITLE;
											break;
										}
									confstruct->ActiveList++;
								break;
							case KEY_LEFT:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
									if ((confstruct->ActiveList - 1) < TITLE)
										{
											confstruct->ActiveList = TEXT;
											break;
										}
									confstruct->ActiveList--;
								break;
							default:
								break;
						}
					swap_active_list_string(confstruct, CursorPos, 1, print_mode);
					return 1;
				break;
			default:
				break;
		}

	return 0;
}



//===============================================================================================

void list_process_paste (CONFSTRUCT *confstruct, FILE **file, int *Marked, int MarkedAction, int MoveDataTo)
{
	FORMULADATA datastruct;		//The structures which will be added to the file at the end
	TEXTDATA textstruct;
	WINDOW *window = NULL;
	HANDLE dialog;
	HANDLE pdown;
	int pdown_choice = 1;
	int Mdt = MoveDataTo;

	GrayOff();

	if (*Marked <= 0)
		{
			DlgMessage("thedb[32]", "CUT or COPY an entry first.", BT_OK, BT_NONE);
			return;
		}

		dialog = DialogNewSimple (120, 50);
		pdown = PopupNew ("" , 0);

			//Add the dialog title
			DialogAddTitle (dialog, "Action", BT_SAVE, BT_CANCEL);
			PopupAddText(pdown, -1, "before", 1);
			PopupAddText(pdown, -1, "after", 2);
			DialogAddText(dialog, 1, 15, "Move marked data");
			DialogAddPulldown(dialog, 2, 25, "Select:", pdown, 0);

		if ((DialogDo (dialog, CENTER, CENTER, NULL, &pdown_choice)) == KEY_ESC)
			return;

	//If "after" has been choosen
	if (pdown_choice == 2)
		Mdt += 1;

	show_process_window (&window, "Copy");

	switch (confstruct->ActiveForm)
		{
			case FORM_FORMULA:
					//Load data
					if ((fmanage(file, ActivePath, FMF_FORMULA_FILE_VERSION, FILE_FORMULA_EXTENSION, &datastruct, sizeof(FORMULADATA), *Marked, FMF_LOAD, 0)) < 0)
						{
							DlgMessage("thedb[21]", "Failed to load!", BT_OK, BT_CANCEL);
							//Free the timer and the window created with show_process_window.
							OSVFreeTimer (1);
						  WinClose (window);
						  HeapFreePtr (window);
							return;
						}
					//Add the new data
					if ((fmanage(file, ActivePath, FMF_FORMULA_FILE_VERSION, FILE_FORMULA_EXTENSION, &datastruct, sizeof(FORMULADATA), Mdt, FMF_ADD_POS, 0)) < 0)
						{
							 DlgMessage("thedb[22]", "Failed to save!", BT_OK, BT_CANCEL);
							//Free the timer and the window created with show_process_window.
							OSVFreeTimer (1);
						  WinClose (window);
						  HeapFreePtr (window);
							return;
						}
				break;
			case FORM_TEXT:
					//Load data
					if ((fmanage(file, ActivePath, FMF_TEXT_FILE_VERSION, FILE_TEXT_EXTENSION, &textstruct, sizeof(TEXTDATA), *Marked, FMF_LOAD, 0)) < 0)
						{
							DlgMessage("thedb[23]", "Failed to load!", BT_OK, BT_CANCEL);
							//Free the timer and the window created with show_process_window.
							OSVFreeTimer (1);
						  WinClose (window);
						  HeapFreePtr (window);
							return;
						}
					//Add the new data
					if ((fmanage(file, ActivePath, FMF_TEXT_FILE_VERSION, FILE_TEXT_EXTENSION, &textstruct, sizeof(TEXTDATA), Mdt, FMF_ADD_POS, 0)) < 0)
						{
							 DlgMessage("thedb[24]", "Failed to save!", BT_OK, BT_CANCEL);
							//Free the timer and the window created with show_process_window.
							OSVFreeTimer (1);
						  WinClose (window);
						  HeapFreePtr (window);
							return;
						}
				break;
			default:
				break;
		}


	if (MarkedAction == MA_CUT)
		{
			//Free the timer and the window created with show_process_window.
			OSVFreeTimer (1);
		  WinClose (window);
		  HeapFreePtr (window);
		  //Show the window again
			show_process_window (&window, "Remove dublicate");

			//Marked + 1 if the new data has been added before MoveDataTo
			if (MoveDataTo < *Marked)
				*Marked = *Marked + 1;

			//Remove data when CUT was choosen
			switch (confstruct->ActiveForm)
				{
					case FORM_FORMULA:
							if ((fmanage(file, ActivePath, FMF_FORMULA_FILE_VERSION, FILE_FORMULA_EXTENSION, &datastruct, sizeof(FORMULADATA), *Marked, FMF_REMOVE, 0)) < 0)
								 DlgMessage("thedb[25]", "Failed to remove!", BT_OK, BT_CANCEL);
						break;
					case FORM_TEXT:
							if ((fmanage(file, ActivePath, FMF_TEXT_FILE_VERSION, FILE_TEXT_EXTENSION, &textstruct, sizeof(TEXTDATA), *Marked, FMF_REMOVE, 0)) < 0)
								 DlgMessage("thedb[26]", "Failed to remove!", BT_OK, BT_CANCEL);
						break;
					default:
						break;
				}
		}


	//Free the timer and the window created with show_process_window.
	OSVFreeTimer (1);
  WinClose (window);
  HeapFreePtr (window);

	return;
}


//===============================================================================================

