// C Source File
// Created 08.03.2006; 16:46:31

 /********************************************************************************
 ********************************************************************************
 * Programm name:		thedb - The handy education database
 * File name:				process_vars.c
 * Function name:		process_vars
 * Header file:			---
 * Other functions:
 *	- read_next_value
 *
 * Short Description: Prepare the window, read the variable names of an expression,
 *										show the dialogs to select a variable name and enter values and
 *										show the solved expression with view2D
 *
 * Description:
 *	-
 *
 ********************************************************************************
 *********************************************************************************/

#include <tigcclib.h>
#include "fmanage.h"

#include "structures.h"
#include "global.h"
#include "calc.h"


//
int read_next_value (char **vars, char *variable);
//

int process_vars (CONFSTRUCT *confstruct, WINDOW *ListWindow, WIN_RECT *ListWinr, int datanr, int *print_mode, int *key)
{
	FILE *datafile = NULL;
	FORMULADATA formuladata;
	HANDLE dialog = H_NULL;
	HANDLE pdown = H_NULL;

	int count = 1;
	int pdown_choice = 1;
	char buffer[MAXLEN_VALUE + 1] = {0};
	char formula[MAXLEN_FORMULA + 1] = {0};
	char v[MAXLEN_VARS + 1] = {0};
	char *vars = v;
	char variable[MAXLEN_VARS + 1] = {0};
	char str[MAXLEN_FORMULA + 20] = {0};
	char solveTo[MAXLEN_VARS + 1] = {0};
	void *test_solveString;

	GrayOff();
	clrscr();

	//Allocate space for the string to test if theres enough memory.
	//That's just because the string is really big, test_solveString will be freed right after.
	if ((test_solveString = malloc(SOLVE_STRING_SIZE)) == NULL)
		{
				DlgMessage("thedb[33]", "Memory!", BT_OK, BT_NONE);
				return 0;
		}
	free(test_solveString);

	char solveString[SOLVE_STRING_SIZE] = {0};


	//Open file and load data
	fmanage(&datafile, ActivePath, FMF_FORMULA_FILE_VERSION, FILE_FORMULA_EXTENSION, &formuladata, sizeof(FORMULADATA), datanr, FMF_NEW_FILE, FMF_NEW_NO_OVERWR);
	fmanage(&datafile, ActivePath, FMF_FORMULA_FILE_VERSION, FILE_FORMULA_EXTENSION, &formuladata, sizeof(FORMULADATA), datanr, FMF_LOAD, 0);


	strcpy(formula, formuladata.formula);		//I thougt I could change the variables of the formula in it's values if they were
																					//given, but that would be hard to implement... because of that, the copy is made.

	if ((read_vars_of_expr(formula, v)) == -1)
		return 0;

	//DlgMessage("The Variables:", v, BT_OK, BT_NONE);

	if (strlen(v) < 1)
		{
			DlgMessage("thedb[34]", "No variables found!", BT_OK, BT_NONE);
			return 0;
		}


	if (*key != KEY_DIAMOND + KEY_F1)
		{
				//--- a dialog with a dropdown list with all variables -------------------------------------
				dialog = DialogNewSimple (LCD_WIDTH - 2, 60);
				pdown = PopupNew ("Variables: " , 0);

				//Add the dialog title
				DialogAddTitle (dialog, "Solve", BT_OK, BT_CANCEL);

				//Loop through the string with the variables and create a pulldown list
				while ((read_next_value(&vars, variable)) == 1)
					{
						PopupAddText(pdown, -1, variable, count);
						count++;
					};


				DialogAddText(dialog, 1, 15, "Choose the variable to solve to:");
					sprintf(str, "solve(%s, ... )", formula);
				DialogAddText(dialog, 1, 25, str);
				DialogAddPulldown(dialog, 60, 35, "...", pdown, 0);

				if ((DialogDo (dialog, CENTER, CENTER, NULL, &pdown_choice)) == KEY_ESC)
					{
						HeapFree (dialog);
						HeapFree(pdown);
						return 0;
					}
				
				HeapFree(pdown);

				//Reset the pointer
				vars = v;
				count = 1;
				//Loop through the string with the variables and search for the selected one
				while ((read_next_value(&vars, variable)) == 1)
					{
						if (count == pdown_choice)
							strcpy(solveTo, variable);

						count++;
					};


			//Prepare solveString
			sprintf(solveString, "solve(%s|", formula);
		}
	else
		{
				//Reset the pointer
				vars = v;
				count = 1;
				//Loop through the string with the variables and search for the last one
				while ((read_next_value(&vars, variable)) == 1)
					{
						strcpy(solveTo, variable);
						count++;
					};

			//Prepare solveString
			sprintf(solveString, "%s|", formula);
		}



	if (*key != KEY_F6)
		{
			//--- the dialog box to enter values for the variables -----------------------------
			//Reset the pointer
			vars = v;
			//Loop through the string with the variables
			while ((read_next_value(&vars, variable)) == 1)
				{
					//Do not show the dialog for the solveTo variable
					if ((strncmp(solveTo, variable, strlen(solveTo))) == 0)
						continue;

					sprintf(buffer, "0.00");

					dialog = DialogNewSimple (LCD_WIDTH - 5, 60);
						DialogAddTitle(dialog, "Enter value", BT_SAVE, BT_CANCEL);
							sprintf(str, "solve(%s, %s)", formula, solveTo);
						DialogAddText(dialog, 1, 20, str);
						DialogAddRequest(dialog, 1, 35, variable, 0, MAXLEN_VALUE, 20);

						if ((DialogDo(dialog, CENTER, CENTER, buffer, NULL)) == KEY_ESC)
							{
								HeapFree (dialog);
								return 0;
							}

						if (strlen(buffer) < 1)
							sprintf(buffer, "0.00");

					if (strlen(solveString) >= SOLVE_STRING_SIZE)
						{
							DlgMessage("thedb[45]", "Memory!", BT_OK, BT_NONE);
							HeapFree (dialog);
							return 0;
						}

					//Add "variable=value and " at the end of the solveString
					strcat(solveString, variable);
					strcat(solveString, "=");
					strcat(solveString, buffer);
					strcat(solveString, " and ");
					memset(buffer, 0, sizeof(buffer));

				};

				//remove the last " and "
				solveString[strlen(solveString) - 5] = '\0';
		}
	else
		{
				//remove the last "|"
				solveString[strlen(solveString) - 1] = '\0';
		}

	if (*key != KEY_DIAMOND + KEY_F1)
		{
			//Finish solveString
			strcat(solveString, ",");
			strcat(solveString, solveTo);
			strcat(solveString, ")");
		}

	HeapFree (dialog);

	//show the formula in the window
	int result = 0;
	*print_mode = 1;
	while ((result = view2d(confstruct, NULL, solveString, ListWindow, ListWinr, &datanr, print_mode)) == 1);
	if ((result == KEY_F1) || (result == KEY_F6) || (result == KEY_DIAMOND + KEY_F1))
		{
			*key = result;
			return 1;
		}

	return 0;
}

//================================================================================

int read_next_value (char **vars, char *variable)
{
	int x = 0;
	int found = 0;


	//Copy letters to "variable" till a "," occurs or break if the end of the string is reached
	while((*vars[0] != ',') && (*vars[0] != '\0'))
		{
			//Copy
			variable[x] = *vars[0];
			//Increase the pointer
			*vars = *vars + 1;
			//Increase the counter
			x++;
			found = 1;
		};


	if (found == 1)
		{
			//Jump over the ","
			*vars = *vars + 1;
			//Finish the string
			variable[x] = '\0';
			return 1;
		}


	return 0;

}

//================================================================================


//================================================================================

//================================================================================

